﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

/* Dieser Eintrag würde es uns ersparen immer "WCFService." zu schreiben 
 * sofern wir auf einen vom WCFService zu Verfügung gestellten Datentyp zugreifen möchten.
 * Um jedoch eine bessere Übersicht für dieses Beispiel beizubehalten, 
 * geben wir diesen Verweis bei jeder Verwendung explizit an. */
// using WCFService; 

// Das Logon Beispiel beschreibt zusätzlich noch, wie man eine Abfrage umsetzen könnte um zu überprüfen ob JavaScript aktiviert ist.

public partial class Logon : System.Web.UI.Page
{
    // Über diese globale Variable steht der WCF-Service in Form eines Proxys zur Verfügung
    private WCFService.AGVIPServiceClient WCFProxy;

    // Konstruktor
    public Logon()
    {
        // WCF-Service Instanz erzeugen (Proxy)
        this.WCFProxy = new WCFService.AGVIPServiceClient();
    }

    // Eventuell beim Laden der Logon.aspx Ansicht zu erledigende Aufgaben könnten wie folgt aussehen
    protected void Page_Load(object sender, EventArgs e)
    {
        // Versionsdaten von AG-VIP SQL über den WCFProxy auslesen
        WCFService.ServiceState serviceData = this.WCFProxy.GetServiceState();

        // Setzen einiger Text-Labels auf der Logon.aspx Ansicht (als Beispiel)
        lblTitel.Text = "AGVIP WCF-Service Login";

        // Dieser Text-Benennungen könnten auch über die Login.aspx Seite direkt eingetragen werden.
        // Normalerweise lässt man über die Sprachendefinition die entsprechenden Bezeichnungen eintragen.
        lblUsername.Text = "Name:";
        lblPassword.Text = "Kennwort:";

        lblAssemblyVersion.Text = "Version: " + serviceData.VersionString;
    }

    protected void btnLogin_Click(object sender, EventArgs e)
    {
        // Beim Laden des Body-Tags der ReadAddresses.aspx werden einige JavaScript-Funktionen aufgerufen.
        // Einer dieser Funktionen ändert einen vordefinierten Wert in einem versteckten Feld.
        // Wenn JavaScript nicht aktiv ist, so wird der Wert im versteckten Feld "HiddenCheckJScript" nicht verändert.
        // Dann leiten wir auf eine aspx Seite weiter um zum Beispiel dem Anwender mitzuteilen, wie er JavaScript aktiviert.
        if (HiddenCheckJScript.Value == "no")
        {
            // Die JSCriptFehler.aspx ist für das Beispiel irrelevant und daher nicht existent.
            // Sie könnte z.B. dem Benutzer in Form eines Textes oder einer Bildabfolge beschreiben, wie man JavaScript aktiviert.
            Response.Redirect("~/JScriptFehler.aspx");
        }

        // Login Versuch mit Rückgabewert der GUID bei Erfolg
        string strGUID = "";
        try
        {
            // Aufruf der "Logon" Funktion vom WCFProxy
            strGUID = this.WCFProxy.Logon(txtUsername.Text, txtPassword.Text);
        }
        catch (System.Exception ex)
        {
            // Wenn beim Versuch die "Logon" Funktion aufzurufen ein Fehler entstanden ist
            // geben wir das in einem Text-Label auf der Ansicht zu erkennen
            lblLoginMessage.Text = ex.Message;
        }

        // Die "Logon" Funktion lieferte erfolgreich eine GUID und signalisiert damit,
        // dass der Anwender sich angemeldet hat und nun eine Session in AG-VIP SQL besitzt.
        if (strGUID != "")
        {
            // Wir speichern die GUID da jeder spätere Funktionsaufruf über den WCFService diese benötigt 
            // und sofern dem Benutzer später noch angezeigt werden soll mit welchem Namen er eingeloggt ist
            // auch noch den Benutzernamen in einer sogenannten Session-Variable.
            // Wenn der <SessionState> Tag nicht in der Web.Config aufgeführt wird,
            // werden automatisch die Standard (Default) Werte verwendet.
            // Näheres zu Session Variablen bzw. des <SessionState> Tags in der Web.Config finden Sie hier:
            // http://msdn.microsoft.com/de-de/library/h6bb9cz9.aspx
            Session["GUID"] = strGUID;
            Session["UserName"] = txtUsername.Text;

            // Weiterleitung auf die nächste Ansicht.
            // Die Willkommen.aspx ist für das Beispiel irrelevant und daher nicht existent.
            Response.Redirect("~/Willkommen.aspx");
        }
    }
}