﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

public partial class Properties : System.Web.UI.Page
{
    // Die Auswahl des Kennzeichenfeldes ist in dem Beispiel fix und wird daher global zur Verfügung gestellt
    private string KennzeichenFeldName = "Kennzeichen";

    private WCFService.AGVIPServiceClient WCFProxy;

    protected void Page_Load(object sender, EventArgs e)
    {
        // WCF-Service Instanz erzeugen (Proxy)
        this.WCFProxy = new WCFService.AGVIPServiceClient();
    }

    // Laden der Kennzeichen des Datensatzes passend zu dem eingegebenen Kurznamen
    protected void btnSuchKurzname_Click(object sender, EventArgs e)
    {
        // Array Liste der zu ladenden Adressfelder
        string[] strArrReadAddress = { "ShortName", KennzeichenFeldName };

        // Array Liste der ausgelesenen Feldwerten
        WCFService.Field[] arrFields = null;

        // Auslesen der Adressdaten siehe "ReadAddresses.aspx.cs"
        // ...

        // Überprüfung auf mehrere Treffer siehe "ReadAddresses.aspx.cs"
        // ...

        // Sonstige Leseoperationen siehe "ReadAddresses.aspx.cs"
        // ...

        // Sämtliche ausgelesenen Daten werden nun Ihren vorgesehenen
        // Feldern auf der Stammdaten-Ansicht zugeordnet
        for (int i = 0; i < arrFields.Length; ++i)
        {
            WCFService.Field field = arrFields[i];
            if (field.GetType() == typeof(WCFService.TextField))
            {
                // siehe "ReadAddresses.aspx.cs"
            }
            else if (field.GetType() == typeof(WCFService.PSetField))
            {
                // Die KennzeichenListenansicht wird sicherheitshalber von alten Werten bereinigt
                lboxKennzeichenAuswahl.Items.Clear();

                WCFService.PSetField PropSetField = (WCFService.PSetField)field;

                // In dem Beispiel eher unnötig, jedoch bei mehreren Kennzeichenfeldern notwendig
                // Wenn das so eben betrachtete Feld das gesuchte Kennzeichenfeld ist so verarbeiten wir dieses
                if (PropSetField.FieldName == KennzeichenFeldName)
                {
                    // Die zuvor geladenen Kennzeichenwerte werden nun in einem Array des Datentyps 'PropertyValue' bereitgestellt
                    WCFService.PropertyValue[] PValues = PropSetField.Value;

                    // Diese Kennzeichenwerte werden nun alle durchlaufen und anschließend in einer ListBox mit deren Name und Beschreibung hinzugefügt.
                    foreach (WCFService.PropertyValue PValue in PValues)
                    {
                        ListItem LItem = new ListItem();
                        LItem.Text = PValue.Name + " / " + PValue.Description;
                        LItem.Value = PValue.Id.ToString();

                        // Das erstellte und befüllte Listen Element (ListItem) der ListBox hinzufügen
                        lboxKennzeichen.Items.Add(LItem);
                    }
                }
            }
        }
       
        // Auswahlliste der Kennzeichenwerte erstellen

        // Auslesen der Category des Kennzeichens "Kennzeichen"
        WCFService.PropertyCategory PropCat = this.WCFProxy.ReadPropertyCategoryByName(Session["GUID"].ToString(), KennzeichenFeldName);

        // Die CategoryId wird für spätere Funktionen in einer Session-Variable gespeichert
        Session["PropCategoryId"] = PropCat.Id;

        // Auslesen der aktiven Kennzeichenwerte (entspricht = 1) ohne einen speziellen Filter, sortiert nach dem Feld "Name"
        WCFService.PropertyValue[] PropValues = this.WCFProxy.ReadPropertyValues(Session["GUID"].ToString(), PropCat.Id, 1, "", "Name");

        // Sämtliche aktive Kennzeichenwerte mit dessen Name und Beschreibung einer Listbox hinzufügen
        foreach (WCFService.PropertyValue PropValue in PropValues)
        {
            ListItem PropValueItem = new ListItem();
            PropValueItem.Text = PropValue.Name + " / " + PropValue.Description;
            PropValueItem.Value = Convert.ToString(PropValue.Id);

            // Das erstellte und befüllte Listen Element (ListItem) der ListBox hinzufügen
            lboxKennzeichenAuswahl.Items.Add(PropValueItem);
        }
        return;
    }

    protected void btnKennzeichen_Click(object sender, EventArgs e)
    {
        // Falls noch keine Kennzeichen Kategorie existiert, geben wir eine Meldung aus
        if (Session["PropCategoryId"].ToString() == "")
        {
            Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler bei der Übernahme", "alert('Bitte zuerst einen Datensatz laden!');", true);
            return;
        }

        // Eine Liste für die Kennzeichenwerte bereitstellen
        List<WCFService.PropertyValue> PropertyValueList = new List<WCFService.PropertyValue>();

        // Durchlaufen aller Kennzeichenwerte der Auswahl Listbox
        foreach (ListItem LItem in lboxKennzeichenAuswahl.Items)
        {
            // Nur wenn ein Kennzeichenwert ausgewählt ist, wird dieser der Liste hinzugefügt
            if (LItem.Selected)
            {
                WCFService.PropertyValue PropVal = new WCFService.PropertyValue();
                PropVal.Id = Convert.ToInt32(LItem.Value);
                PropertyValueList.Add(PropVal);
            }
        }

        // Wir erzeugen ein PSetField welches der Container der Kennzeichenwerte darstellt.
        // In diesem Beispiel wären das die Kennzeichenwerte für die Kennzeichenliste "Kennzeichen"
        WCFService.PSetField PropertyField = new WCFService.PSetField();

        // Zur eindeutigen Zuordnung benötigen wir die Kategorie und den Feldnamen oder die Id
        PropertyField.Category = Convert.ToInt32(Session["PropCategoryId"]);
        PropertyField.FieldName = KennzeichenFeldName;

        // Dem Container wird die KennzeichenwerteListe übergeben.
        PropertyField.Value = PropertyValueList.ToArray();

        // letztendlich muss der befüllte Container für eine Abfrage in einen 'Field' Array übergeben werden
        WCFService.Field[] fldList = { PropertyField };
        // Die aktuell geöffnete Adresse soll bearbeitet werden
        try
        {
            // Alle Felder im 'fldList' Array übernehmen/aktuallisieren
            // Wenn kein Kennzeichenwert ausgewählt wurde, so würden alle dem Datensatz bereits zugeordneten Kennzeichenwerte entfernt werden
            this.WCFProxy.UpdateAddress(Session["GUID"].ToString(), Convert.ToInt32(DDLAddressSets.SelectedValue), Convert.ToInt32(Session["IdAddress"].ToString()), fldList);
        }
        catch (System.Exception ex)
        {
            Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Speichern", "alert('" + ex.Message + "');", true);
            return;
        }
        return;
    }
}