﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

// Dieses Beispiel beschreibt einiges zusätzlich anstatt nur das einfache Auslesen von Adressdaten.
// Zusätzlich werden noch Abfragen beschrieben welche mehrere Treffer bei der Suche nach der Adresse behandeln
// und wie man einen Datensatz nur lesend öffnet sofern dieser bereits durch einen anderen Benutzer gesperrt sein sollte.
// Vor dem Auslesen sollte wie ebenfalls hier zusätzlich beschrieben, eine Abfrage erfolgen ob bereits eine andere Adresse geöffnet (gesperrt) wurde,
// welche dann anschließend geschlossen (freigegeben) wird.

public partial class ReadAddresses : System.Web.UI.Page
{
    private WCFService.AGVIPServiceClient WCFProxy;

    protected void Page_Load(object sender, EventArgs e)
    {
        // WCF-Service Instanz erzeugen (Proxy)
        this.WCFProxy = new WCFService.AGVIPServiceClient();
    }

    protected void btnSuchKurzname_Click(object sender, EventArgs e)
    {
        // Falls bereits eine Adresse geöffnet ist, geben wir diese wieder frei
        if (Session["IdAddress"].ToString() != "0")
        {
            try
            {
                // Wenn eine Adresse oder ein Ticket mit dem Parameter "lck=true" geladen wird, so ist diese Adresse bzw. dieses Ticket für andere Benutzer gesperrt.
                // Diese Sperrung wird durch einen expliziten Aufruf der "UnlockAddress" Funktion wieder freigegeben.
                // Da beim Öffnen eines Tickets, eigentlich die Adresse geöffnet und gesperrt wird, 
                // muss über die selbe Funktion "UnlockAddress" die Adresse wieder freigegeben werden.
                // Sofern die Adresse nicht explizit freigegeben wurde, so wird es spätestens beim Ausführen der "Logoff" Funktion wieder freigegeben.
                // Falls jedoch die "Logoff" Funktion nicht aufgerufen werden sollte, würde die Adresse oder das Ticket solange gesperrt bleiben,
                // wie die Session noch existiert (ca. 15 Minuten maximal).
                this.WCFProxy.UnlockAddress(Session["GUID"].ToString(), Convert.ToInt32(DDLAddressSets.SelectedValue), Convert.ToInt32(Session["IdAddress"]));
            }
            catch (System.Exception ex)
            {
                // Falls die Adresse aus unerklärlichen Gründen nicht freigegeben werden konnte
                // wird dies dem Benutzer mitgeteilt, jedoch der Suchvorgang nicht abgebrochen
                Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Freigeben der Adresse", "alert('" + ex.Message + "');", true);
            }
        }

        // Liste der auszulesenden Adressdaten (Felder)
        string[] strArrReadAddresses = { "Id" };

        WCFService.Record[] arrRecord = null;
        
        // Suchen aller Adressen über einen Vergleich des Kurznamen
        try
        {
            arrRecord = this.WCFProxy.ReadAddresses(Session["GUID"].ToString(), Convert.ToInt32(DDLAddressSets.SelectedValue), "ShortName=='" + txtKurzname.Text + "*'", "", strArrReadAddresses);
        }
        catch (System.Exception ex)
        {
            Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Lesen", "alert('" + ex.Message + "');", true);
            return;
        }

        // Falls über den Kurznamen keine Adresse gefunden werden konnte
        // wird dies dem Benutzer mitgeteilt und der restliche Suchvorgang abgebrochen
        if (arrRecord == null)
        {
            Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Lesen", "alert('Keine Daten gefunden!');", true);
            return;
        }
        else
        {
            // Ergab die Suche eventuell mehrere Treffer?
            if (arrRecord.Length > 1)
            {
                // speichern der ausgewählten AdressTabellenId
                Session["AddressSetId"] = DDLAddressSets.SelectedValue;
                string Ids = "";
                int i = 1;

                // erstellen einer Stringfolge aller Adressen Ids des Suchergebnisses
                foreach (WCFService.Record rec in arrRecord)
                {
                    if (i < arrRecord.Length)
                    {
                        Ids += rec.Id + " ";
                    }
                    else
                    {
                        Ids += rec.Id;
                    }
                    ++i;
                }

                // Beispiel für eine Auswahlliste von Adressen:
                // Script zum Öffnen eines neuen Pop-Up Fensters
                // Die Stringfolge von Ids wird als Paramater mit übergeben (RecordIds)
                // Diese könnten jedoch auch in einer Session Variable hinterlegt werden
                string PopUpTrefferListeScript = "<script language=JavaScript>";
                PopUpTrefferListeScript += "window.open('PopUpTrefferListe.aspx?RecordIds=" + Ids + "', 'Ausgabe', 'width=400,height=315,left=950,top=5');";
                PopUpTrefferListeScript += "</script>";

                // Die PopUpTrefferListeScript.aspx ist für dieses Beispiel irrelevant und daher nicht existent
                // Sie stellt desweiteren auch nur eine Auswahlliste der mehreren Auswahlmöglichkeiten von Adressen dar (Simple Listbox).
                Page.ClientScript.RegisterStartupScript(this.GetType(), "Ausgabe", PopUpTrefferListeScript);

                // Die PopUpTrefferListeScript.aspx soll eine Liste von Adressen darstellen aus denen der Anwender eine Auswählen muss.
                // Nach der Auswahl geht es im Prinzip wie bei einer einzelnen Adresse weiter um diese zu laden (siehe nachfolgende else-Schleife 'Die Suche ergab genau einen Treffer')
            }
            else
            {
                // Die Suche ergab genau einen Treffer
                // somit kann sofort der Datensatz (Adresse) geladen werden

                // Array Liste der gewünschten Adressfelder
                string[] strArrReadAddress = { "NameFirst", "NameLast" };
                WCFService.Field[] arrFields = null;

                // Die Id der Adresse wird für eine spätere Verwendung in einer Session Variable gespeichert
                Session["IdAddress"] = arrRecord[0].Id;

                // Auslesen der Adressdaten (Felder)
                try
                {
                    // Die Adresse wird exklusiv geöffnet (lck=true). Dadurch bekommen andere Benutzer keinen Schreibzugriff während diese Adresse gesperrt ist.
                    arrFields = this.WCFProxy.ReadAddress(Session["GUID"].ToString(), Convert.ToInt32(DDLAddressSets.SelectedValue), arrRecord[0].Id, true, strArrReadAddress);
                }
                catch (System.Exception ex)
                {
                    Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Lesen", "alert('" + ex.Message + "');", true);
                    return;
                }

                // Überprüfen ob beim Auslesen der Adresse auch Daten zurückgegeben wurden
                if (arrFields == null)
                {
                    // Es wird ein zweites Mal versucht die Adresse zu öffnen
                    // jedoch dieses mal nur mit Lesezugriff (Ohne Lock)
                    try
                    {
                        // Die Adresse wird nicht exklusiv geöffnet (lck=false). Dieser Zugriff funktioniert in der Regel immer, sofern die Adresse existiert (Nur Lese-Zugriff). 
                        arrFields = this.WCFProxy.ReadAddress(Session["GUID"].ToString(), Convert.ToInt32(DDLAddressSets.SelectedValue), arrRecord[0].Id, false, strArrReadAddress);
                    }
                    catch (System.Exception ex)
                    {
                        Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Laden", "alert('" + ex.Message + "');", true);
                        return;
                    }

                    // Falls immer noch keine Werte zurückgegeben wurden
                    // scheint die Adresse nicht zu existieren
                    if (arrFields == null)
                    {
                        Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Laden", "alert('Keine Daten gefunden!');", true);
                        return;
                    }
                    else
                    {
                        // Falls jetzt doch Werte zurückgegeben wurden
                        // Ist die Adresse momentan von einem anderen Benutzer gesperrt
                        Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Laden", "alert('Adresse gesperrt! Nur Lesezugriff möglich!');", true);
                    }
                }

                // Die ausgelesenen Daten werden nun Ihren vorgesehenen Feldern auf der Ansicht zugeordnet
                for (int i = 0; i < arrFields.Length; ++i)
                {
                    WCFService.Field field = arrFields[i];
                    if (field.GetType() == typeof(WCFService.TextField))
                    {
                        WCFService.TextField txtField = (WCFService.TextField)field;
                        switch (txtField.FieldName)
                        {
                            case "NameFirst":
                                txtVorname.Text = txtField.Value;
                                break;
                            case "NameLast":
                                txtNachname.Text = txtField.Value;
                                break;
                        }
                    }
                }
                return;
            }
        }
    }
}