﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

public partial class ReadTickets : System.Web.UI.Page
{
    private WCFService.AGVIPServiceClient WCFProxy;

    protected void Page_Load(object sender, EventArgs e)
    {
        // WCF-Service Instanz erzeugen (Proxy)
        this.WCFProxy = new WCFService.AGVIPServiceClient();

        // Die Liste der Adresstabellen und Projekttabellen bei einem Postback nicht erneut erstellen
        if (!Page.IsPostBack)
        {
            // Befüllen der Adresstabellenliste siehe ReadAddresses.aspx.cs
            // ...

            // Erstellen einer Projekttabellenliste für den angemeldeten Benutzer
            WCFService.Project[] ProjektTabellenListe = this.WCFProxy.ReadProjects(Session["GUID"].ToString(), Convert.ToInt32(DDLAddressSets.SelectedValue), "IsSystem<>1", "Name");
            
            // Übernahme der Projekttabellen in die DropDownListe
            foreach (WCFService.Project ProjektTabelle in ProjektTabellenListe)
            {
                ListItem ProjecttblItem = new ListItem();
                ProjecttblItem.Text = ProjektTabelle.Name;
                ProjecttblItem.Value = Convert.ToString(ProjektTabelle.Id);
                DDLProjects.Items.Add(ProjecttblItem);
            }

        }
    }

    // Öffnen/Laden der Adresse über den "Suchen" Button siehe ReadAddresses.aspx
    // ...
       
    protected void ReadTickets_Click(object sender, EventArgs e)
    {
        // Liste der auszulesenden Ticketdaten (Felder)
        string[] strArrReadTickets = { "Id" };
        WCFService.Record[] arrTickets = null;

        if (Session["IdAddress"].ToString() != "0")
        {
            arrTickets = this.WCFProxy.ReadTickets(Session["GUID"].ToString(), Convert.ToInt32(DDLProjects.SelectedValue), Convert.ToInt32(DDLAddressSets.SelectedValue), "IdAddress=" + Session["IdAddress"].ToString(), "", strArrReadTickets);
        }
        else
        {
            Page.ClientScript.RegisterStartupScript(this.GetType(), "Ticket öffnen / laden", "alert('Zuerst eine Adresse öffnen / laden!');", true);
            return;
        }

        if (arrTickets == null)
        {
            Page.ClientScript.RegisterStartupScript(this.GetType(), "Ticket öffnen / laden", "alert('Keine Tickets gefunden!');", true);
            return;
        }
        else
        {
            // Ergab die Suche mehrere Treffer?
            if (arrTickets.Length > 1)
            {
                // speichern der ausgewählten ProjektTabellenId
                Session["ProjectId"] = DDLProjects.SelectedValue;
                string Ids = "";
                int i = 1;

                // erstellen einer Stringfolge aller Ticket Ids des Suchergebnisses
                foreach (WCFService.Record rec in arrTickets)
                {
                    if (i < arrTickets.Length)
                    {
                        Ids += rec.Id + " ";
                    }
                    else
                    {
                        Ids += rec.Id;
                    }
                    ++i;
                }

                // Script zum Öffnen eines neuen Pop-Up Fensters
                // Die Stringfolge von Ids wird als Paramater mit übergeben (RecordIds)
                // Diese könnten jedoch auch in einer Session Variable hinterlegt werden
                string PopUpTrefferListeScript = "<script language=JavaScript>";
                PopUpTrefferListeScript += "window.open('PopUpTrefferListe.aspx?RecordIds=" + Ids + "', 'Ausgabe', 'width=400,height=315,left=950,top=5');";
                PopUpTrefferListeScript += "</script>";

                // Die PopUpTrefferListeScript.aspx ist für dieses Beispiel irrelevant und daher nicht existent
                Page.ClientScript.RegisterStartupScript(this.GetType(), "Ausgabe", PopUpTrefferListeScript);

                // Die PopUpTrefferListeScript.aspx soll eine Liste von Tickets darstellen aus denen der Anwender eine Auswählen muss.
                // Nach der Auswahl geht es im Prinzip wie bei einem einzelnen Ticket weiter um dieses zu laden (siehe nachfolgende else-Schleife 'Die Suche ergab genau einen Treffer')
            }
            else
            {
                // Die Suche ergab genau einen Treffer
                // somit kann sofort das Ticket geladen werden

                // Array Liste der gewünschten Ticketfelder
                string[] strArrReadTicket = { "TimeTicket", "TicketDescription" };
                WCFService.Field[] arrFields = null;

                Session["IdTicket"] = arrTickets[0].Id;

                // Auslesen der Ticketdaten (Felder)
                try
                {
                    // Das Ticket wird exklusiv geöffnet (lck=true). Dadurch bekommen andere Benutzer keinen Schreibzugriff während dieses Ticket gesperrt ist.
                    arrFields = this.WCFProxy.ReadTicket(Session["GUID"].ToString(), Convert.ToInt32(DDLProjects.SelectedValue), arrTickets[0].Id, true, strArrReadTicket);
                }
                catch (System.Exception ex)
                {
                    Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Lesen", "alert('" + ex.Message + "');", true);
                    return;
                }

                // Überprüfen ob beim Auslesen des Tickets auch Daten zurückgegeben wurden
                if (arrFields == null)
                {
                    // Es wird ein zweites Mal versucht das Ticket zu öffnen
                    // jedoch dieses mal nur mit Lesezugriff (Ohne Lock)
                    try
                    {
                        // Das Ticket wird nicht exklusiv geöffnet (lck=false). Dieser Zugriff funktioniert in der Regel immer, sofern die Adresse existiert (Nur Lese-Zugriff). 
                        arrFields = this.WCFProxy.ReadTicket(Session["GUID"].ToString(), Convert.ToInt32(DDLProjects.SelectedValue), arrTickets[0].Id, false, strArrReadTicket);
                    }
                    catch (System.Exception ex)
                    {
                        Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Laden", "alert('" + ex.Message + "');", true);
                        return;
                    }

                    // Falls immer noch keine Werte zurückgegeben wurden existiert das Ticket nicht
                    if (arrFields == null)
                    {
                        Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Laden", "alert('Keine Daten gefunden!');", true);
                        return;
                    }
                    else
                    {
                        // Falls jetzt doch Werte zurückgegeben wurden
                        // Ist das Ticket momentan von einem anderen Benutzer gesperrt
                        Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Laden", "alert('Adresse gesperrt! Nur Lesezugriff möglich!');", true);
                    }

                    // Ausgelesenen Daten werden nun Ihren vorgesehenen Feldern auf der Ansicht zugeordnet
                    for (int i = 0; i < arrFields.Length; ++i)
                    {
                        WCFService.Field field = arrFields[i];
                        if (field.GetType() == typeof(WCFService.TextField))
                        {
                            WCFService.TextField txtField = (WCFService.TextField)field;
                            switch (txtField.FieldName)
                            {
                                case "TimeTicket":
                                    txtTimeTicket.Text = txtField.Value;
                                    break;
                                case "TicketDescription":
                                    txtTicketDesc.Text = txtField.Value;
                                    break;
                            }
                        }
                    }
                    return;
                }
            }
        }
    }
}