﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

// In diesem Beispiel wird beschrieben, wie man SubQueries verwendet
// Zum Beispiel um eine Liste von verkauften Produkten abzubilden, welche in einem bestimmten PLZ-Bereich verkauft wurden (Die Produkte sind hinterlegt in einer Zusatztabelle)
// Desweiteren werden noch kleinere Benutzerfehleingaben bei der PLZ-Bereich Eingabe behandelt

public partial class SubQuery : System.Web.UI.Page
{
    private WCFService.AGVIPServiceClient WCFProxy;

    protected void Page_Load(object sender, EventArgs e)
    {
        // WCF-Service Instanz erzeugen (Proxy)
        this.WCFProxy = new WCFService.AGVIPServiceClient();
    }

    protected void ReadRelation_Click(object sender, EventArgs e)
    {
        string strSubQuery1 = "";
        string strSubQuery2 = "";

        // Benutzereingaben Abfrage bezüglich des Inhalts des PLZ-Bereichs
        if (txtPLZBis.Text == "")
        {
            if (txtPLZVon.Text != "")
            {
                txtPLZBis.Text = txtPLZVon.Text;
            }
            else
            {
                Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler bei der Eingabe", "alert('Bitte zuerst einen PLZ-Bereich angeben!');", true);
                return;
            }
        }
        else if (Convert.ToInt32(txtPLZVon.Text) > Convert.ToInt32(txtPLZBis.Text))
        {
            Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler bei der Eingabe", "alert('Der PLZ-Bereich VON darf nicht größer sein als der von BIS!');", true);
            return;
        }

        try
        {
            // Die Parameter werden in diesem Beispiel über Session Variablen ausgelesen welche bereits in Beispielen zuvor befüllt wurden bzw. die PLZ Daten über Felder auf der SubQuery.aspx ermittelt.
            strSubQuery1 = this.WCFProxy.CreateSubQueryOnAddressRecords(Session["GUID"].ToString(), Convert.ToInt32(Session["IdAddressSet"]), "(ZipCode>=" + txtPLZVon.Text + ")&(" + "ZipCode<=" + txtPLZBis.Text + ")");
        }
        catch (System.Exception ex)
        {
            Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Lesen", "alert('" + ex.Message + "');", true);
            return;
        }

        WCFService.Relation rlProdukte = null;
        try
        {
            // Die Parameter werden in diesem Beispiel Fix hinterlegt bzw. über Session Variablen ausgelesen welche bereits in Beispielen zuvor befüllt wurden.
            // Als fixer Wert wird hier der Name einer Zusatztabelle angegeben (Produkte). 
            // Dieser Parameter sollte über ein Eingabe- bzw. Auswahlfeld vom Benutzer oder automatisch bestimmt werden.
            rlProdukte = this.WCFProxy.ReadRelationByName(Session["GUID"].ToString(), Convert.ToInt32(Session["IdAddressSet"]), Convert.ToInt32(Session["IdProject"]), "Produkte");

        }
        catch (System.Exception ex)
        {
            Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Lesen", "alert('" + ex.Message + "');", true);
            return;
        }

        try
        {
            // Übergabe der ZusatztabellenId mit der IdAddress Einschränkung der zuvor getätigten SubQuerieOnAddressRecords Funktion.
            strSubQuery2 = this.WCFProxy.CreateSubQueryOnRelationRecords(Session["GUID"].ToString(), rlProdukte.Id, "IdAddress=" + strSubQuery1);

        }
        catch (System.Exception ex)
        {
            Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Lesen", "alert('" + ex.Message + "');", true);
            return;
        }

        // Zusammenführung der auszulesenden Felder Der Zusatztabelle in einem String Array
        string[] fldList = { "Seriennummer", "Artikel", "Anzahl", "ZustellDatum" };
        WCFService.Record[] RecRelationRecords = null;

        try
        {
            // Wir lesen die Zusatztabelle mit den über die SubQueries zusammengetragenen Einschränkungen aus.
            RecRelationRecords = this.WCFProxy.ReadRelation(Session["GUID"].ToString(), rlProdukte.Id, 0, "IdAddress=" + strSubQuery2, "", fldList);

        }
        catch (System.Exception ex)
        {
            Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Lesen", "alert('" + ex.Message + "');", true);
            return;
        }

        // Überprüfung ob es überhaupt Rückgabewerte gab
        if (RecRelationRecords != null)
        {
            foreach (WCFService.Record RecRelationRecord in RecRelationRecords)
            {
                TableRow row = new TableRow();
                foreach (WCFService.Field fldField in RecRelationRecord.Fields)
                {
                    // Die Abfragen nach dem Feldnamen sind in diesem Beispiel überflüssig,
                    // da es keine Zuordnung der Werte zu einem Feld gibt, sondern diese nacheinander
                    // in den Tabellespalten eingetragen werden
                    TableCell cell = new TableCell();
                    if (fldField.GetType() == typeof(WCFService.TextField))
                    {
                        // Textfeld (Seriennummer)
                        WCFService.TextField txtField = (WCFService.TextField)fldField;

                        cell.Text = txtField.Value;
                        row.Cells.Add(cell);
                        tblRelation.Rows.Add(row);
                    }
                    else if (fldField.GetType() == typeof(WCFService.DateTimeField))
                    {
                        // Datumsfeld (Zustelldatum)
                        WCFService.DateTimeField dtField = (WCFService.DateTimeField)fldField;

                        cell.Text = dtField.Value.ToString();
                        row.Cells.Add(cell);
                        tblRelation.Rows.Add(row);
                    }
                    else if (fldField.GetType() == typeof(WCFService.IntegerField))
                    {
                        // Zahlenfeld (Anzahl)
                        WCFService.IntegerField FieldId = (WCFService.IntegerField)fldField;

                        cell.Text = FieldId.Value.ToString();
                        row.Cells.Add(cell);
                        tblRelation.Rows.Add(row);
                    }
                    else if (fldField.GetType() == typeof(WCFService.PValField))
                    {
                        // Kennzeichenfeld (Artikel)
                        WCFService.PValField KennzeichenfeldWert = (WCFService.PValField)fldField;

                        // Bei einem Kennzeichenfeld müssen wir die Werte in eine PropertyValue Variable übergeben
                        WCFService.PropertyValue propVal = KennzeichenfeldWert.Value;

                        // Über die PropertyValue Variable kann auf die Kennzeichenwerte zugegriffen werden z.B. die Beschreibung
                        cell.Text = propVal.Description;
                        row.Cells.Add(cell);
                        tblRelation.Rows.Add(row);
                    }
                }
            }
        }
        else
        {
            Page.ClientScript.RegisterStartupScript(this.GetType(), "Fehler beim Lesen", "alert('Keine Daten zu den Einschränkungen gefunden!');", true);
            return;
        }
    }
}